@@ -1,3 +1,4 @@
+import os
 import datetime
 import pyhcup
 
@@ -102,6 +103,7 @@ class File(models.Model):
     #partner file for actual loading, if this is a content file
     loadfile = models.ForeignKey('self', blank=True, default=None, null=True)
     
+    
     def _hr_size(self):
         """Returns size in MB"""
         if self.size_on_disk:
@@ -110,22 +112,41 @@ class File(models.Model):
             return None
     hr_size = property(_hr_size)
     
+    
     def _est_rows(self):
         if self.file_type.compression == 'NONE':
             full_size = self.hr_size
         else:
             full_size = 5*self.hr_size
         return 9850*full_size
-    est_rows = property(_est_rows)    
+    est_rows = property(_est_rows)
+    
+    
+    def open(self):
+        c = self.file_type.compression
+        
+        if c == 'NONE':
+            # uncompressed
+            handle = open(self.full_path)
+        elif c == 'ZIP':
+            # pkzip compression
+            handle = pyhcup.hachoir.openz(os.path.dirname(self.full_path), self.filename)
+        else:
+            raise Exception("Unknown compression type %s; unable to generate file handle." % c)
+        
+        return handle
+    
     
     class Meta:
         verbose_name = 'file'
         verbose_name_plural = 'files'
         ordering = ['file_type__source', '-file_type__compression', 'state', 'year', 'filename']
     
+    
     def get_absolute_url(self):
         return reverse('djhcup_staging.views.file_detail', kwargs={'file_id': self.pk})
     
+    
     def __unicode__(self):
         return 'File: %s (%s)' % (self.filename, self.file_type)
 
@@ -277,6 +298,7 @@ class ImportQueue(models.Model):
         return self.file.est_rows/(rows_per_hour)
     est_runtime = property(_est_runtime)
     
+    
     class Meta:
         verbose_name = 'enqueued item'
         verbose_name_plural = 'enqueued items'
@@ -301,6 +323,75 @@ class ImportQueue(models.Model):
             return False
     
     
+    def record_columns(self):
+        tls = self.batch.destination.source.top_level_source.abbreviation
+        cols_saved = 0
+        
+        if tls == 'HCUP':
+            meta = pyhcup.sas.meta_from_sas(self.file.loadfile.full_path)
+            augmented = pyhcup.meta.augment(meta)
+            
+            # store the columns for future reference
+            for k, v in augmented.T.to_dict().iteritems():
+                #col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
+                col = Column(
+                    parent_file=self.file,
+                    name=v['field'],
+                    col_type=v['data_type'].upper(),
+                    col_scale=v['scale'],
+                    col_precision=v['length'],
+                    informat=v['informat'],
+                    )
+                col.save()
+                cols_saved += 1
+        
+        elif tls == 'PUDF':
+            meta = pyhcup.tx.meta_from_txt(self.file.loadfile.full_path)
+            augmented = pyhcup.meta.augment(meta, dialect='PUDF')
+            
+            # store the columns for future reference
+            for k, v in augmented.T.to_dict().iteritems():
+                # col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
+                col = Column(
+                    parent_file=self.file,
+                    name=v['field'],
+                    col_type=v['data_type'].upper(),
+                    col_scale=v['scale'],
+                    col_precision=v['length'],
+                    # informat=v['informat'],
+                    )
+                col.save()
+                cols_saved += 1
+                
+        else:
+            raise Exception('No loadfile parser associated with top-level source %s' % tls)
+        
+        return cols_saved
+    
+    
+    def get_meta(self):
+        tls = self.batch.destination.source.top_level_source.abbreviation
+        
+        if tls == 'HCUP':
+            loader = pyhcup.sas.meta_from_sas
+        
+        elif tls == 'PUDF':
+            loader = pyhcup.tx.meta_from_txt
+        
+        else:
+            raise Exception('No loadfile parser associated with top-level source %s' % tls)
+        
+        
+        loadfile = self.file.loadfile
+        
+        if loadfile is None:
+            raise Exception('No loadfile associated with file %s'% self.file)
+        else:
+            meta = loader(loadfile.full_path)
+        
+        return meta
+    
+    
     def __unicode__(self):
         return "Enqueued item %s [%s]: %s, destination %s" % (self.pk, self.status, self.file.filename, self.batch.destination.name)
 
