@@ -40,6 +40,10 @@ from celery.result import AsyncResult
 
 HACHOIR_DB_ENTRY = 'djhcup'
 
+# check to see if psycopg2 is how we're going
+DB_DEF = settings.DATABASES[HACHOIR_DB_ENTRY]
+DB_BACKEND = DB_DEF['ENGINE'].split('.')[-1]
+
 @celery.shared_task
 @tx.atomic # turn off autocommit
 def reset_dead():
@@ -116,7 +120,7 @@ def reset_dead():
 
 
 @celery.shared_task(bind=True, ignore_result=True)
-def batch_import(self, batch_id, load_chunksize=5, placeholder_representation="%s"):
+def batch_import(self, batch_id):
     """Attempts to import the indicated batch"""
     
     batch = ImportBatch.objects.get(pk=batch_id)
@@ -131,11 +135,13 @@ def batch_import(self, batch_id, load_chunksize=5, placeholder_representation="%
     if batch.complete:
         response = 'Batch %i was previously completed at %s, and therefore will not be run' % (batch.pk, batch.complete)
         logger.error(response)
+        batch.fail()
         return False
     
     elif enqueued.count() < 1:
         response = 'Batch %i has no items enqueued, and therefore will not be run' % batch.pk
         logger.warning(response)
+        batch.fail()
         return True
     
     #make sure the items in the batch have the same data source as the destination table
@@ -143,6 +149,7 @@ def batch_import(self, batch_id, load_chunksize=5, placeholder_representation="%
         response = 'One or more items enqueued in batch %i has a different data source than was used to generate the master table intended to receive data (destination master table data source is %s), and therefore the batch will not be run' % (batch.pk, batch.destination.source)
         logger.error(response)
         [logger.error('%s has data source %s' % (x, x.file.file_type.source)) for x in enqueued if x.file.file_type.source != destination.source]
+        batch.fail()
         return False
         
     else:
@@ -155,68 +162,30 @@ def batch_import(self, batch_id, load_chunksize=5, placeholder_representation="%
         batch.save()
         logger.info('Begin batch %s' % (batch))
         
-        # decide now whether to use import_file in a chain (old way)
-        # or pg_import_file in a group (new hotness)
-        dsabbr = destination.source.abbreviation
-        
-        # FROM HERE BELOW, SPLIT OFF PSYCOPG2 VS NOT
-        
-        # check to see if psycopg2 is how we're going
-        db_def = settings.DATABASES[HACHOIR_DB_ENTRY]
-        backend = db_def['ENGINE'].split('.')[-1]
-        
-        if 'psycopg2' in backend and dsabbr.upper() in ['CORE', 'BASE']:
-            # do it Rockapella
-            job = celery.group(
-                pg_import_file.si(load.pk) for load in enqueued
-                )
-            
-            # fire the group
-            job()
-        
-        else:
-            # make a chain from task signatures that churn through each enqueued object
-            job = celery.chain(
-                import_file.si(load.pk) for load in enqueued
-                )
-            
-            # this could, alternatively, be a task group
-            
-            # fire the chain
-            job()
+        job = celery.group(
+            import_dispatch.si(load.pk) for load in enqueued
+            )
+        job()
         
         # put a task into background for periodic check that the batch has completed
-        batch_complete.apply_async((batch.pk,), countdown=60)
+        batch_complete.apply_async((batch.pk,), countdown=600)
         
         return True
 
 
-"""
-
-try:
-    module = importlib.import_module(x)
-    installed[x] = True
-except ImportError:
-    installed[x] = False
-
-"""
-
-
 @celery.shared_task(bind=True, ignore_result=True)
-def import_file2(self, importqueue_pk, insert_size=5, slice_size=1000):
+def import_dispatch(self, importqueue_pk, insert_size=5, slice_size=1000):
     load = ImportQueue.objects.get(pk=importqueue_pk)
-    load.celery_task_id = self.request.id
-    load.save()
     
     destination = load.batch.destination
     similar_load_qs = ImportQueue.objects.filter(batch__destination=destination, file=load.file)
-    similar_completed = similar_load_qs.filter(complete__isnull=False)
+    similar_completed = similar_load_qs.filter(complete__isnull=False, status='SUCCESS')
     similar_started = similar_load_qs.filter(start__isnull=False)
     
 
     if similar_completed.count() > 0:
         #appears to already have been imported. skip this one
-        logger.warning('It looks like <%s> was already imported to <%s> at %s; skipping for now' % (load, destination, similar_completed[0].complete))
+        logger.warning('It looks like <%s> was already imported successfully to <%s> at %s; skipping for now' % (load, destination, similar_completed[0].complete))
         load.fail()
         return False
     
@@ -228,346 +197,181 @@ def import_file2(self, importqueue_pk, insert_size=5, slice_size=1000):
 
     else:
         # proceed with making the file happen
-        load.status = 'IN PROCESS'
-        load.start = datetime.datetime.utcnow().replace(tzinfo=utc)
+        load.status = 'QUEUED'
         load.save()
         
-        logger.info('Begin import of %s to %s' % (load, destination))
+        logger.info('Queued for dispatch: %s' % (load))
         
-        tls = destination.source.top_level_source.abbreviation
-        if tls == 'HCUP':
-            meta = pyhcup.sas.meta_from_sas(load.file.loadfile.full_path)
-            augmented = pyhcup.meta.augment(meta)
-            
-            # store the columns for future reference
-            for k, v in augmented.T.to_dict().iteritems():
-                #col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
-                col = Column(
-                    parent_file=load.file,
-                    name=v['field'],
-                    col_type=v['data_type'].upper(),
-                    col_scale=v['scale'],
-                    col_precision=v['length'],
-                    informat=v['informat'],
-                    )
-                col.save()
-        
-        elif tls == 'PUDF':
-            meta = pyhcup.tx.meta_from_txt(load.file.loadfile.full_path)
-            augmented = pyhcup.meta.augment(meta, dialect='PUDF')
-            
-            # store the columns for future reference
-            for k, v in augmented.T.to_dict().iteritems():
-                # col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
-                col = Column(
-                    parent_file=load.file,
-                    name=v['field'],
-                    col_type=v['data_type'].upper(),
-                    col_scale=v['scale'],
-                    col_precision=v['length'],
-                    # informat=v['informat'],
-                    )
-                col.save()
-                
-        table = destination.name
-        schema = destination.schema
-        state = load.file.state.abbreviation
-        year = load.file.year
         dsabbr = destination.source.abbreviation
         
         # FROM HERE BELOW, SPLIT OFF PSYCOPG2 VS NOT
         
-        # check to see if psycopg2 is how we're going
-        db_def = settings.DATABASES[HACHOIR_DB_ENTRY]
-        backend = db_def['ENGINE'].split('.')[-1]
-        
-        if 'psycopg2' in backend and dsabbr.upper() in ['CORE', 'BASE']:
+        if ('psycopg2' in DB_BACKEND and (
+                ((dsabbr.upper() in ['CORE', 'BASE']) or
+                 (dsabbr.upper() in ['CHGS'] and meta_is_long(load.get_meta()))
+                    )
+                )
+            ):
             # do it Rockapella
+            logger.info('Sending to pg_import_file: %s' % (load))
             pg_import_file.delay(load.pk)
             return True
         
         else:
-            # use the old chunky way
-            if load.file.file_type.compression == 'NONE':
-                # uncompressed
-                handle = open(load.file.full_path)
-            elif load.file.file_type.compression == 'ZIP':
-                # pkzip compression
-                handle = pyhcup.hachoir.openz(os.path.dirname(load.file.full_path), load.file.filename)
-            
-            # make a generator to yield one slice_size slice of the file in question at a time
-            reader = pyhcup.parser.read(handle, meta, chunksize=slice_size)
-            
-            # take 5
-            # loop through slices and dispatch chunker.chunks() for each
-            # this is preferred to a chain so that it doesn't wait to start until after all the signatures are generated
-            result_lst = []
-            for df in reader:
-                # everything we insert will always have a state and year associated with it
-                # if they aren't already in the frame, add them from the passed values
-                # (which should in turn be from the loadfile object)
-                if 'state' not in df.columns.map(lambda x: str(x).lower()):
-                    df['state'] = state
-                
-                if 'year' not in df.columns.map(lambda x: str(x).lower()):
-                    df['year'] = year
-                
-                cleaned_df = replace_sentinels(df)
-                
-                if dsabbr == 'CHGS' and not meta_is_long(meta):
-                    # this is a wide charges file
-                    # convert before chunking it out
-                    cleaned_df = df_wtl(cleaned_df, category='CHGS')
-                
-                result_lst.append(chunker(
-                        cleaned_df,
-                        table,
-                        schema,
-                        insert_size
-                        )
-                    )
-            
-            import_complete.apply_async((result_lst, load.pk), countdown=10)
-            return True
+            logger.info('Sending to chunk_import_file: %s' % (load))
+            chunk_import_file.delay(load.pk)
 
 
 @celery.shared_task(bind=True, ignore_result=True)
-def pg_import_file(self, importqueue_pk):
+def pg_import_file(self, importqueue_pk, drop_intermediate_tables=False):# change to True for production
     """Assumes psycopg2 availability.
     """
     
     load = ImportQueue.objects.get(pk=importqueue_pk)
-    
+    load.status = 'RECEIVED'
     load.celery_task_id = self.request.id
     load.save()
     
-    destination = load.batch.destination
-    similar_load_qs = ImportQueue.objects.filter(batch__destination=destination, file=load.file)
-    similar_completed = similar_load_qs.filter(complete__isnull=False)
-    similar_started = similar_load_qs.filter(start__isnull=False)
+    logger.info('Received for import with psycopg2: %s' % (load))
     
-    if similar_completed.count() > 0:
-        #appears to already have been imported. skip this one
-        logger.warning('It looks like <%s> was already imported to <%s> at %s; skipping for now' % (load, destination, similar_completed[0].complete))
-        load.fail()
-        return False
-    elif similar_started.count() > 0:
-        #appears to already have been imported. skip this one
-        logger.warning('It looks like an import was already started for importing <%s> to <%s> at %s; skipping for now' % (load, destination, similar_started[0].start))
-        load.fail()
-        return False
-    else:
-        # proceed with making the file happen
-        load.status = 'IN PROCESS'
-        load.start = datetime.datetime.utcnow().replace(tzinfo=utc)
-        load.save()
-        
-        logger.info('Begin import of %s to %s' % (load, destination))
-        
-        tls = destination.source.top_level_source.abbreviation
-        if tls == 'HCUP':
-            meta = pyhcup.sas.meta_from_sas(load.file.loadfile.full_path)
-            augmented = pyhcup.meta.augment(meta)
-            
-            # store the columns for future reference
-            for k, v in augmented.T.to_dict().iteritems():
-                #col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
-                col = Column(
-                    parent_file=load.file,
-                    name=v['field'],
-                    col_type=v['data_type'].upper(),
-                    col_scale=v['scale'],
-                    col_precision=v['length'],
-                    informat=v['informat'],
-                    )
-                col.save()
-        
-        elif tls == 'PUDF':
-            meta = pyhcup.tx.meta_from_txt(load.file.loadfile.full_path)
-            augmented = pyhcup.meta.augment(meta, dialect='PUDF')
-            
-            # store the columns for future reference
-            for k, v in augmented.T.to_dict().iteritems():
-                # col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
-                col = Column(
-                    parent_file=load.file,
-                    name=v['field'],
-                    col_type=v['data_type'].upper(),
-                    col_scale=v['scale'],
-                    col_precision=v['length'],
-                    # informat=v['informat'],
-                    )
-                col.save()
-                
-        table = destination.name
-        schema = destination.schema
-        state = load.file.state.abbreviation
-        year = load.file.year
-        dsabbr = destination.source.abbreviation
+    destination = load.batch.destination
 
-        target_file = load.file
-        
-        c = target_file.file_type.compression
-        
-        if c == 'NONE':
-            # uncompressed
-            handle = open(target_file.full_path)
-        elif c == 'ZIP':
-            # pkzip compression
-            handle = pyhcup.hachoir.openz(os.path.dirname(target_file.full_path), target_file.filename)    
-        
-        # establish connection directly with psycopg2
-        import psycopg2
-        db_def = settings.DATABASES[HACHOIR_DB_ENTRY]
-        cnxn = psycopg2.connect(
-            host=db_def['HOST'],
-            port=db_def['PORT'],
-            user=db_def['USER'],
-            password=db_def['PASSWORD'],
-            database=db_def['NAME'],
-            )
-        
-        # get helper functions
-        from pyhcup.db import pg_rawload, pg_staging, pg_shovel, pg_drop
-        
-        # let 'er rip
-        rawload = pg_rawload(cnxn, handle, meta)
-        staging = pg_staging(cnxn, rawload, meta, state, year)
-        
-        fields = [x for x in meta.field if x.lower() not in ['state', 'year']]
-        fields += ['state', 'year']
-        
-        affected_rows = pg_shovel(cnxn, staging[0], table, fields)
-        
+    # proceed with making the file happen
+    load.status = 'IN PROCESS'
+    load.start = datetime.datetime.utcnow().replace(tzinfo=utc)
+    load.save()
+    
+    logger.info('Begin import: %s' % (load))
+    
+    cols_saved = load.record_columns()
+    if cols_saved > 0:
+        logger.info('Columns saved: %s for %s' % (cols_saved, load))
+    else:
+        logger.info('No columns saved for %s' % (load))
+    
+    table = destination.name
+    schema = destination.schema
+    state = load.file.state.abbreviation
+    year = load.file.year
+    dsabbr = destination.source.abbreviation
+    
+    handle = load.file.open()
+    
+    # establish connection directly with psycopg2
+    import psycopg2
+    cnxn = psycopg2.connect(
+        host=DB_DEF['HOST'],
+        port=DB_DEF['PORT'],
+        user=DB_DEF['USER'],
+        password=DB_DEF['PASSWORD'],
+        database=DB_DEF['NAME'],
+        )
+    
+    # get helper functions
+    from pyhcup.db import pg_rawload, pg_staging, pg_shovel, pg_drop
+    
+    # let 'er rip
+    meta = load.get_meta()
+    
+    rawload = pg_rawload(cnxn, handle, meta)
+    
+    if load.file.file_type.source.abbreviation in pyhcup.parser.LONG_MAPS.keys():
+        pk_fields = None
+    else:
+        pk_fields = ['key', 'state', 'year']
+    
+    staging = pg_staging(cnxn, rawload, meta, state, year, pk_fields=pk_fields)
+    
+    fields = [x for x in meta.field if x.lower() not in ['state', 'year']]
+    fields += ['state', 'year']
+    
+    affected_rows = pg_shovel(cnxn, staging[0], table, fields)
+    
+    if drop_intermediate_tables:
         pg_drop(cnxn, rawload)
         pg_drop(cnxn, staging[0])
-        
-        # affected_rows will evaluate un-True in the case of any troubles
-        if affected_rows:
-            load.success()
-            logger.info('Import complete: %s (%s rows)' % (load, affected_rows))
-            return True
-        else:
-            load.fail()
-            logger.error('Import failed: %s' % load)
-            return False
+    
+    # affected_rows will evaluate un-True in the case of any troubles
+    if affected_rows:
+        load.success()
+        logger.info('Import complete: %s (%s rows)' % (load, affected_rows))
+        return True
+    else:
+        load.fail()
+        logger.error('Import failed: %s' % load)
+        return False
 
 
 @celery.shared_task(bind=True, ignore_result=True)
-def import_file(self, importqueue_pk, insert_size=5, slice_size=1000):
+def chunk_import_file(self, importqueue_pk, insert_size=5, slice_size=1000):
+    """Imports a file with a tangled web of task distribution in order to effect parallelization.
+    """
+    
     load = ImportQueue.objects.get(pk=importqueue_pk)
+    load.status = 'RECEIVED'
     load.celery_task_id = self.request.id
     load.save()
     
+    logger.info('Received for import with celery task chunking: %s' % (load))
+    
     destination = load.batch.destination
-    similar_load_qs = ImportQueue.objects.filter(batch__destination=destination, file=load.file)
-    similar_completed = similar_load_qs.filter(complete__isnull=False)
-    similar_started = similar_load_qs.filter(start__isnull=False)
     
-
-    if similar_completed.count() > 0:
-        #appears to already have been imported. skip this one
-        logger.warning('It looks like <%s> was already imported to <%s> at %s; skipping for now' % (load, destination, similar_completed[0].complete))
-        load.fail()
-        return False
+    # proceed with making the file happen
+    load.status = 'IN PROCESS'
+    load.start = datetime.datetime.utcnow().replace(tzinfo=utc)
+    load.save()
     
-    elif similar_started.count() > 0:
-        #appears to already have been imported. skip this one
-        logger.warning('It looks like an import was already started for importing <%s> to <%s> at %s; skipping for now' % (load, destination, similar_started[0].start))
-        load.fail()
-        return False
-
+    logger.info('Begin import: %s' % (load))
+    
+    cols_saved = load.record_columns()
+    if cols_saved > 0:
+        logger.info('Columns saved: %s for %s' % (cols_saved, load))
     else:
-        # proceed with making the file happen
-        load.status = 'IN PROCESS'
-        load.start = datetime.datetime.utcnow().replace(tzinfo=utc)
-        load.save()
+        logger.info('No columns saved for %s' % (load))
+    
+    table = destination.name
+    schema = destination.schema
+    state = load.file.state.abbreviation
+    year = load.file.year
+    dsabbr = destination.source.abbreviation
+    
+    handle = load.file.open()
+    meta = load.get_meta()
+    
+    # make a generator to yield one slice_size slice of the file in question at a time
+    reader = pyhcup.parser.read(handle, meta, chunksize=slice_size)
+
+    # take 5
+    # loop through slices and dispatch chunker.chunks() for each
+    # this is preferred to a chain so that it doesn't wait to start until after all the signatures are generated
+    result_lst = []
+    for df in reader:
+        # everything we insert will always have a state and year associated with it
+        # if they aren't already in the frame, add them from the passed values
+        # (which should in turn be from the loadfile object)
+        if 'state' not in df.columns.map(lambda x: str(x).lower()):
+            df['state'] = state
         
-        logger.info('Begin import of %s to %s' % (load, destination))
+        if 'year' not in df.columns.map(lambda x: str(x).lower()):
+            df['year'] = year
         
-        tls = destination.source.top_level_source.abbreviation
-        if tls == 'HCUP':
-            meta = pyhcup.sas.meta_from_sas(load.file.loadfile.full_path)
-            augmented = pyhcup.meta.augment(meta)
-            
-            # store the columns for future reference
-            for k, v in augmented.T.to_dict().iteritems():
-                #col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
-                col = Column(
-                    parent_file=load.file,
-                    name=v['field'],
-                    col_type=v['data_type'].upper(),
-                    col_scale=v['scale'],
-                    col_precision=v['length'],
-                    informat=v['informat'],
-                    )
-                col.save()
+        cleaned_df = replace_sentinels(df)
         
-        elif tls == 'PUDF':
-            meta = pyhcup.tx.meta_from_txt(load.file.loadfile.full_path)
-            augmented = pyhcup.meta.augment(meta, dialect='PUDF')
-            
-            # store the columns for future reference
-            for k, v in augmented.T.to_dict().iteritems():
-                # col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
-                col = Column(
-                    parent_file=load.file,
-                    name=v['field'],
-                    col_type=v['data_type'].upper(),
-                    col_scale=v['scale'],
-                    col_precision=v['length'],
-                    # informat=v['informat'],
-                    )
-                col.save()
+        if dsabbr == 'CHGS' and not meta_is_long(meta):
+            # this is a wide charges file
+            # convert before chunking it out
+            cleaned_df = df_wtl(cleaned_df, category='CHGS')
         
-        if load.file.file_type.compression == 'NONE':
-            # uncompressed
-            handle = open(load.file.full_path)
-        elif load.file.file_type.compression == 'ZIP':
-            # pkzip compression
-            handle = pyhcup.hachoir.openz(os.path.dirname(load.file.full_path), load.file.filename)
-        
-        # make a generator to yield one slice_size slice of the file in question at a time
-        reader = pyhcup.parser.read(handle, meta, chunksize=slice_size)
-        
-        table = destination.name
-        schema = destination.schema
-        state = load.file.state.abbreviation
-        year = load.file.year
-        dsabbr = destination.source.abbreviation
-        
-        
-        # take 5
-        # loop through slices and dispatch chunker.chunks() for each
-        # this is preferred to a chain so that it doesn't wait to start until after all the signatures are generated
-        result_lst = []
-        for df in reader:
-            # everything we insert will always have a state and year associated with it
-            # if they aren't already in the frame, add them from the passed values
-            # (which should in turn be from the loadfile object)
-            if 'state' not in df.columns.map(lambda x: str(x).lower()):
-                df['state'] = state
-            
-            if 'year' not in df.columns.map(lambda x: str(x).lower()):
-                df['year'] = year
-            
-            cleaned_df = replace_sentinels(df)
-            
-            if dsabbr == 'CHGS' and not meta_is_long(meta):
-                # this is a wide charges file
-                # convert before chunking it out
-                cleaned_df = df_wtl(cleaned_df, category='CHGS')
-            
-            result_lst.append(chunker(
-                    cleaned_df,
-                    table,
-                    schema,
-                    insert_size
-                    )
+        result_lst.append(chunker(
+                cleaned_df,
+                table,
+                schema,
+                insert_size
                 )
-        
-        import_complete.apply_async((result_lst, load.pk), countdown=10)
-        return True
+            )
+    
+    import_complete.apply_async((result_lst, load.pk), countdown=10)
+    return True
 
 
 @celery.shared_task
@@ -656,7 +460,7 @@ def import_complete(self, result_lst, importqueue_pk):
         raise self.retry(exc=Exception('Import not yet complete: %s' % iq))
 
 
-@celery.shared_task(bind=True, max_retries=None, default_retry_delay=60)# default_retry_delay=60)
+@celery.shared_task(bind=True, max_retries=None, default_retry_delay=600)# default_retry_delay=60)
 def batch_complete(self, batch_pk):
     """Checks to see if each item in the batch has a completion timestamp in db, and updates batch db entry to reflect success/failure of overall batch.
     """
