@@ -464,7 +464,7 @@ def batch_detail(request, batch_id):
     return render(request, template, context)
 
 
-def batch_import(request, batch_id, load_chunksize=5, placeholder_representation="%s"):
+def batch_import(request, batch_id):
     """Schedules the indicated batch_id for import using Celery"""
     logger.debug('Batch %s import requested' % batch_id)
     
@@ -472,7 +472,7 @@ def batch_import(request, batch_id, load_chunksize=5, placeholder_representation
     from djhcup_staging import tasks
     
     logger.info('Attempting to call a Celery task for importing batch_id %i' % (int(batch_id)))
-    tresult = tasks.batch_import.delay(batch_id, load_chunksize, placeholder_representation)
+    tresult = tasks.batch_import.delay(batch_id)
     
     response = "Batch has been dispatched to Celery in thread %s; its completion state is %s" % (tresult, tresult.state)
     logger.info(response)
